/**
 * @(#) BabelFishTranslator.java
 */

package com.softsky.transj.babelfish;

import java.net.*;
import java.util.Vector;
import java.io.*;
import com.softsky.transj.translator.ITranslator;
import com.softsky.transj.IExceptionHandler;
import com.softsky.transj.translator.Direction;
import com.softsky.transj.translator.Language;
import java.util.*;
import java.nio.*;
import java.nio.charset.*;

public class BabelFishTranslator implements ITranslator, Serializable {
    
    private Hashtable words = new Hashtable(4096);
    private IExceptionHandler exceptionHandler = null;
    private boolean rememberText  = false;
    
        /* (non-Javadoc)
         * @see com.softsky.transj.translator.ITranslator#translate(java.lang.String)
         */
    public String translate(Direction direction, String text, boolean rememberText, int connectionTimeout) {
        this.rememberText = rememberText;
        if(rememberText && words.containsKey(text))
            return translateLocal(direction, text);
        else
            return translateOnline(direction, text );
    }
    private String translateLocal(Direction direction, String text ) {
        return (String)words.get(text);
    }
    
    private String translateOnline(Direction direction, String text ) {
        try{
            URL babelFishURL = new URL("http://babelfish.altavista.com/babelfish/tr");
            HttpURLConnection conn = (HttpURLConnection)babelFishURL.openConnection();
            conn.setRequestMethod("POST");
            conn.setRequestProperty("User-Agent", "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.6) Gecko/20040510");
            conn.setRequestProperty("Accept", "text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,image/jpeg,image/gif;q=0.2,*/*;q=0.1");
            conn.setRequestProperty("Accept-Language", "ru,en-us;q=0.7,en;q=0.3");
            conn.setRequestProperty("Accept-Encoding", "gzip,deflate");
            conn.setRequestProperty("Accept-Charset", "utf-8;q=0.7,*;q=0.7");
            conn.setRequestProperty("Keep-Alive", "300");
            conn.setRequestProperty("Proxy-Connection", "close");
            conn.setRequestProperty("Referer", "http://babelfish.av.com/");
            conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            
            conn.setDoOutput(true);
            conn.getOutputStream().write(("doit=done&intl=1&tt=urltext&urltext="+text+"&lp=" +
            direction.getFromLanguage().getShortName().toLowerCase()
            + "_" +
            direction.getToLanguage().getShortName().toLowerCase()).getBytes());
            
            InputStream is = conn.getInputStream();
            String enc = conn.getHeaderField("Content-Encoding");
            if(enc != null && enc.equals("gzip"))
                is = new java.util.zip.GZIPInputStream(is);
            
            ByteArrayOutputStream os = new ByteArrayOutputStream(4096);
            int i;
            byte [] buf = new byte[4096];
            while((i = is.read(buf)) > -1)
                os.write(buf, 0, i);
            
            conn.disconnect();
            is.close();
            Charset charset = Charset.forName("UTF-8");
            
            CharsetDecoder decoder = charset.newDecoder()
            .onMalformedInput(CodingErrorAction.REPLACE)
            .onUnmappableCharacter(CodingErrorAction.REPLACE);
            
            ByteBuffer bb = ByteBuffer.wrap(os.toByteArray());
            String result = decoder.decode(bb).toString();
            
            String searchStr = "<td bgcolor=white class=s><div style=padding:10px;>";
            int idx = result.indexOf(searchStr);
            if(idx == -1)
                return null;
            int start = idx + searchStr.length();
            String translated = result.substring(start, result.indexOf("</div>", start));
            translated = unescapeHTML(translated, 0);
            if(rememberText)
                words.put(text, translated);
            return translated;
            
        }catch(MalformedURLException e) {
            exceptionHandler.handleException(e);
        }
        catch(IOException e) {
            exceptionHandler.handleException(e);
        }
        return null;
    }
    public Direction[] getSupportedDirections( ) {
        Vector vecDirections = new Vector();
        
        vecDirections.add(new Direction(Language.ZH, Language.EN));
        vecDirections.add(new Direction(Language.ZT, Language.EN));
        vecDirections.add(new Direction(Language.EN, Language.ZH));
        vecDirections.add(new Direction(Language.EN, Language.ZT));
        vecDirections.add(new Direction(Language.EN, Language.NL));
        vecDirections.add(new Direction(Language.EN, Language.FR));
        vecDirections.add(new Direction(Language.EN, Language.DE));
        vecDirections.add(new Direction(Language.EN, Language.EL));
        vecDirections.add(new Direction(Language.EN, Language.IT));
        vecDirections.add(new Direction(Language.EN, Language.JA));
        vecDirections.add(new Direction(Language.EN, Language.KO));
        vecDirections.add(new Direction(Language.EN, Language.PT));
        vecDirections.add(new Direction(Language.EN, Language.RU));
        vecDirections.add(new Direction(Language.EN, Language.ES));
        vecDirections.add(new Direction(Language.NL, Language.EN));
        vecDirections.add(new Direction(Language.NL, Language.FR));
        vecDirections.add(new Direction(Language.FR, Language.EN));
        vecDirections.add(new Direction(Language.FR, Language.DE));
        vecDirections.add(new Direction(Language.FR, Language.EL));
        vecDirections.add(new Direction(Language.FR, Language.IT));
        vecDirections.add(new Direction(Language.FR, Language.PT));
        vecDirections.add(new Direction(Language.FR, Language.NL));
        vecDirections.add(new Direction(Language.FR, Language.ES));
        vecDirections.add(new Direction(Language.DE, Language.EN));
        vecDirections.add(new Direction(Language.DE, Language.FR));
        vecDirections.add(new Direction(Language.EL, Language.EN));
        vecDirections.add(new Direction(Language.EL, Language.FR));
        vecDirections.add(new Direction(Language.IT, Language.EN));
        vecDirections.add(new Direction(Language.IT, Language.FR));
        vecDirections.add(new Direction(Language.JA, Language.EN));
        vecDirections.add(new Direction(Language.KO, Language.EN));
        vecDirections.add(new Direction(Language.PT, Language.EN));
        vecDirections.add(new Direction(Language.PT, Language.FR));
        vecDirections.add(new Direction(Language.RU, Language.EN));
        vecDirections.add(new Direction(Language.ES, Language.EN));
        vecDirections.add(new Direction(Language.ES, Language.FR));
        
        Direction [] supportedDirections = new Direction[vecDirections.size()];
        vecDirections.copyInto(supportedDirections);
        return supportedDirections;
    }
    
    
    public static final String unescapeHTML(String s, int f){
        String [][] escape =
        {{  "&lt;"     , "<" } ,
         {  "&gt;"     , ">" } ,
         {  "&amp;"    , "&" } ,
         {  "&quot;"   , "\"" } ,
         {  "&agrave;" , "а" } ,
         {  "&Agrave;" , "А" } ,
         {  "&acirc;"  , "в" } ,
         {  "&auml;"   , "д" } ,
         {  "&Auml;"   , "Д" } ,
         {  "&Acirc;"  , "В" } ,
         {  "&aring;"  , "е" } ,
         {  "&Aring;"  , "Е" } ,
         {  "&aelig;"  , "ж" } ,
         {  "&AElig;"  , "Ж" } ,
         {  "&ccedil;" , "з" } ,
         {  "&Ccedil;" , "З" } ,
         {  "&eacute;" , "й" } ,
         {  "&Eacute;" , "Й" } ,
         {  "&egrave;" , "и" } ,
         {  "&Egrave;" , "И" } ,
         {  "&ecirc;"  , "к" } ,
         {  "&Ecirc;"  , "К" } ,
         {  "&euml;"   , "л" } ,
         {  "&Euml;"   , "Л" } ,
         {  "&iuml;"   , "п" } ,
         {  "&Iuml;"   , "П" } ,
         {  "&ocirc;"  , "ф" } ,
         {  "&Ocirc;"  , "Ф" } ,
         {  "&ouml;"   , "ц" } ,
         {  "&Ouml;"   , "Ц" } ,
         {  "&oslash;" , "ш" } ,
         {  "&Oslash;" , "Ш" } ,
         {  "&szlig;"  , "Я" } ,
         {  "&ugrave;" , "щ" } ,
         {  "&Ugrave;" , "Щ" } ,
         {  "&ucirc;"  , "ы" } ,
         {  "&Ucirc;"  , "Ы" } ,
         {  "&uuml;"   , "ь" } ,
         {  "&Uuml;"   , "Ь" } ,
         {  "&nbsp;"   , " " } ,
         {  "&reg;"    , "\u00a9" } ,
         {  "&copy;"   , "\u00ae" } ,
         {  "&euro;"   , "\u20a0" } };
         int i, j, k, l ;
         
         i = s.indexOf("&", f);
         if (i > -1) {
             j = s.indexOf(";" ,i);
             // --------
             // we don't start from the beginning
             // the next time, to handle the case of
             // the &amp;
             // thanks to Pieter Hertogh for the bug fix!
             f = i + 1;
             // --------
             if (j > i) {
                 // ok this is not most optimized way to
                 // do it, a StringBuffer would be better,
                 // this is left as an exercise to the reader!
                 String temp = s.substring(i , j + 1);
                 // search in escape[][] if temp is there
                 k = 0;
                 while (k < escape.length) {
                     if (escape[k][0].equals(temp)) break;
                     else k++;
                 }
                 if (k < escape.length) {
                     s = s.substring(0 , i) + escape[k][1] + s.substring(j + 1);
                     return unescapeHTML(s, f); // recursive call
                 }
                 else
                     if(temp.startsWith("&#")){
                         // parse hex code
                         String szCode = temp.substring(2, temp.length() - 1);
                         int iCode = Integer.parseInt(szCode, 10);
                         char [] chars = new char[1];
                         chars[0] = (char)iCode;
                         s = s.substring(0 , i) + new String(chars) + s.substring(j + 1);
                         return unescapeHTML(s, f); // recursive call
                     }
             }
         }
         return s;
    }
    
    /**
     * Getter for property exceptionHandler.
     * @return Value of property exceptionHandler.
     */
    public IExceptionHandler getExceptionHandler() {
        return exceptionHandler;
    }
    
    /**
     * Setter for property exceptionHandler.
     * @param exceptionHandler New value of property exceptionHandler.
     */
    public void setExceptionHandler(IExceptionHandler exceptionHandler) {
        this.exceptionHandler = exceptionHandler;
    }
    
    public void clearCache() {
        words.clear();
    }
    
}
