/**
 * @(#) Options.java
 */

package com.softsky.transj.ui.swt;


import java.util.*;
import java.io.*;
import com.softsky.transj.translator.Direction;
import com.softsky.transj.translator.Language;
import com.softsky.transj.TransJ;

public class Options implements Serializable {
    private Direction direction;
    private boolean rememberText = false;
    private boolean scanAndReverse = false;
    private boolean sendAllErrors = false;
    private int timeout = 30;
    private String browserLaunch = null;
    private Vector activeLanguages;
    
    
    
    public Options( ){
        direction = new Direction(Language.EN, Language.RU);
        activeLanguages = new java.util.Vector();
        // setting default values
        rememberText = true;
        scanAndReverse = true;
        sendAllErrors = true;
        browserLaunch = getDefaultBrowserLaunch();
        // enabling all languages
        for(Enumeration langs = Language.getAllLanguages();langs.hasMoreElements();){
            Language lang = (Language)langs.nextElement();
            setLanguageActive(lang, true);
        }
    }
    
    /**
     * Getter for property direction.
     * @return Value of property direction.
     */
    public Direction getDirection( ) {
        return direction;
    }
    
    /**
     * Setter for property direction.
     * @param direction New value of property direction.
     */
    public void setDirection( Direction direction ) {
        this.direction = direction;
    }
    
    /**
     * Getter for property rememberText.
     * @return Value of property rememberText.
     */
    public boolean isRememberText( ) {
        return rememberText;
    }
    
    /**
     * Setter for property rememberText.
     * @param rememberText New value of property rememberText.
     */
    public void setRememberText( boolean rememberText ) {
        this.rememberText = rememberText;
    }
    
    /**
     * Getter for property scanAndReverse.
     * @return Value of property scanAndReverse.
     */
    public boolean isScanAndReverse( ) {
        return scanAndReverse;
    }
    
    /**
     * Setter for property scanAndReverse.
     * @param scanAndReverse New value of property scanAndReverse.
     */
    public void setScanAndReverse( boolean scanAndReverse ) {
        this.scanAndReverse = scanAndReverse;
    }
    
    /**
     * Getter for property sendAllErrors.
     * @return Value of property sendAllErrors.
     */
    public boolean isSendAllErrors( ) {
        return sendAllErrors;
    }
    
    /**
     * Setter for property sendAllErrors.
     * @param sendAllErrors New value of property sendAllErrors.
     */
    public void setSendAllErrors( boolean sendAllErrors ) {
        this.sendAllErrors = sendAllErrors;
    }
    
    
    public void setLanguageActive( Language language, boolean active ) {
        if(active && (activeLanguages.indexOf(language) == -1))
            activeLanguages.add(language);
        else
            if(!active)
                activeLanguages.remove(language);
    }
    
    public boolean isLanguageActive(Language lang){
        return activeLanguages.contains(lang);
    }
    
    
    public Enumeration getAllLanguages( ) {
        return Language.getAllLanguages();
    }
    
    /**
     * Getter for property browserLaunch.
     * @return Value of property browserLaunch.
     */
    public java.lang.String getBrowserLaunch() {
        if(browserLaunch == null)
            browserLaunch = getDefaultBrowserLaunch();
        return browserLaunch;
    }    
    
    /**
     * Setter for property browserLaunch.
     * @param browserLaunch New value of property browserLaunch.
     */
    public void setBrowserLaunch(java.lang.String browserLaunch) {
        this.browserLaunch = browserLaunch;
    }
    
    /**
     * Getter for property timeout.
     * @return Value of property timeout.
     */
    public int getTimeout() {
        return timeout;
    }
    
    /**
     * Setter for property timeout.
     * @param timeout New value of property timeout.
     */
    public void setTimeout(int timeout) {
        this.timeout = timeout;
    }
    
    private String getDefaultBrowserLaunch(){
        if(TransJ.OSNAME.indexOf("Windows") > -1)
            return "rundll32 url.dll,FileProtocolHandler %url";
        else
            if (TransJ.OSNAME.equals("Linux"))
            return "mozilla \"openURL %url\"";
        return "";
    }
    
}
